@@ -5,14 +5,14 @@
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.36.1")
+(defparameter *autoconf-version* "5.37.3")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
-(defparameter *autoconf-host* "powerpc-unknown-openbsd5.8")
+(defparameter *autoconf-host* "alpha-unknown-openbsd5.9")
 (defparameter *autoconf-win32* "false")
 (defparameter *autoconf-ld-flags* "")
  
-(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.36.1/maxima-5.36.1")
+(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.37.3/maxima-5.37.3")
 (defparameter *maxima-default-layout-autotools* "true")
