@@ -11,77 +11,119 @@
 (defun memq (x lis)
   (member x lis :test #'eq))
 
-(eval-when
-    #+gcl (compile load)
-    #+ecl (compile load eval)
-    #-(or gcl ecl) (:compile-toplevel :load-toplevel)
+;;this will make operators which declare the type and result of numerical operations
+(eval-when (:compile-toplevel :load-toplevel :execute)
 
-    ;;this will make operators which
-    ;;declare the type and result of numerical operations
+  (defmacro def-op (name arg-type op &optional return-type)
+    `(setf (macro-function ',name)
+           (make-operation ',arg-type ',op ',return-type)))
 
-    (defmacro def-op (name arg-type op &optional return-type)
-      `(setf (macro-function ',name)
-	     (make-operation ',arg-type ',op ',return-type)))
-
-    ;;make very sure .type .op and .return are not special!!
-    (defun make-operation (.type .op .return)
-      (or .return (setf .return .type))
-      #'(lambda (bod env)
-	  (declare (ignore env))
-	  (loop for v in (cdr bod)
-	     when (eq t .type) collect v into body
-	     else
-	     collect `(the , .type ,v) into body
-	     finally (setq body `(, .op ,@body))
+  ;;make very sure .type .op and .return are not special!!
+  (defun make-operation (.type .op .return)
+    (or .return (setf .return .type))
+    #'(lambda (bod env)
+        (declare (ignore env))
+        (loop for v in (cdr bod)
+           when (eq t .type) collect v into body
+           else
+           collect `(the , .type ,v) into body
+           finally (setq body `(, .op ,@body))
 	     (return
 	       (if (eq t .return)
 		   body
 		   `(the , .return ,body))))))
 
-    #+fix-debug
-    (progn
-      ;; these allow running of code and they print out where the error occurred
+  ;; these allow running of code and they print out where the error occurred
+  #+fix-debug
+  (progn
+    (defvar *dbreak* t)
 
-      (defvar *dbreak* t)
+    (defun chk-type (lis na typ sho)
+      (unless (every #'(lambda (v) (typep v typ)) lis)
+        (format t "~%Bad call ~a types:~a" (cons na sho) (mapcar #'type-of lis))
+        (when *dbreak*
+          (break "hi"))))
 
-      (defun chk-type (lis na typ sho)
-	(unless (every #'(lambda (v) (typep v typ)) lis)
-	  (format t "~%Bad call ~a types:~a" (cons na sho)
-		  (mapcar #'type-of lis))
-	  (when *dbreak*
-	    (break "hi"))))
+    (defmacro def-op (name arg-type old)
+      `(defmacro ,name (&rest l)
+         `(progn
+            (chk-type (list ,@l) ',',name ',',arg-type ',l)
+            (,',old ,@l)))))
 
-      (defmacro def-op (name arg-type old)
-	`(defmacro ,name (&rest l)
-	   `(progn (chk-type (list ,@l) ',',name ',',arg-type ',l)
-		   (,',old ,@l)))))
-
-    (def-op f+ fixnum +)
-    (def-op f* fixnum *)
-    (def-op f- fixnum -)
-    (def-op f1- fixnum 1-)
-    (def-op f1+ fixnum 1+)
-    (def-op quotient t quot)
-    (def-op // t quot))
+  (def-op f+ fixnum +)
+  (def-op f* fixnum *)
+  (def-op f- fixnum -)
+  (def-op f1- fixnum 1-)
+  (def-op f1+ fixnum 1+)
+  (def-op quotient t quot))
 
 ;;this is essentially what the quotient is supposed to do.
 
-(defun quot (a &rest b)
-  (cond ((null b)
-	 (quot 1 a))
-	((null (cdr b))
-	 (setq b (car b))
-	 (cond ((and (integerp a) (integerp b))
-		(values (truncate a b)))
-	       (t
-		(/ a b))))
-	(t (apply #'quot (quot a (car b)) (cdr b)))))
+(declaim (inline quot))
+(defun quot (a b)
+  (if (and (integerp a) (integerp b))
+      (truncate a b)
+      (/ a b)))
 
 (defmacro status (option &optional item)
   (cond ((equal (symbol-name option) (symbol-name '#:feature))
 	 `(member ,(intern (string item) (find-package 'keyword)) *features*))
 	((equal option 'gctime) 0)))
 
+#+(or scl allegro)
+(defun string<$ (str1 str2)
+  "Compare string, but flip the case for maxima variable names to maintain
+  the same order irrespective of the lisp case mode."
+  (declare (string str1 str2))
+  (cond (#+scl (eq ext:*case-mode* :lower)
+	 #+allegro (eq excl:*current-case-mode* :case-sensitive-lower)
+	 (let ((str1l (length str1))
+	       (str2l (length str2)))
+	   (cond ((and (> str1l 1) (char= (aref str1 0) #\$)
+		       (> str2l 1) (char= (aref str2 0) #\$))
+		  (flet ((case-flip (str)
+			   (let ((some-upper nil)
+				 (some-lower nil))
+			     (dotimes (i (length str))
+			       (let ((ch (schar str i)))
+				 (when (lower-case-p ch)
+				   (setf some-lower t))
+				 (when (upper-case-p ch)
+				   (setf some-upper t))))
+			     (cond ((and some-upper some-lower)
+				    nil)
+				   (some-upper
+				    :downcase)
+				   (some-lower
+				    :upcase)))))
+		    (let ((flip1 (case-flip str1))
+			  (flip2 (case-flip str2)))
+		      (do ((index 1 (1+ index)))
+			  ((or (>= index str1l) (>= index str2l))
+			   (if (= index str1l) index nil))
+			(let ((ch1 (aref str1 index))
+			      (ch2 (aref str2 index)))
+			  (cond ((and (eq flip1 :downcase) (both-case-p ch1))
+				 (setf ch1 (char-downcase ch1)))
+				((and (eq flip1 :upcase) (both-case-p ch1))
+				 (setf ch1 (char-upcase ch1))))
+			  (cond ((and (eq flip2 :downcase) (both-case-p ch2))
+				 (setf ch2 (char-downcase ch2)))
+				((and (eq flip2 :upcase) (both-case-p ch2))
+				 (setf ch2 (char-upcase ch2))))
+			  (unless (char= ch1 ch2)
+			    (return (if (char< ch1 ch2)
+					index
+					nil))))))))
+		 (t
+		  (string< str1 str2)))))
+	(t
+	 (string< str1 str2))))
+;;;
+#-(or scl allegro)
+(defun string<$ (str1 str2)
+  (string< str1 str2))
+
 ;;numbers<strings<symbols<lists<?
 (defun alphalessp (x y)
   (cond ((numberp x)
@@ -97,7 +139,7 @@
 		(let ((nx (symbol-name x))
 		      (ny (symbol-name y)))
 		  (declare (string nx ny))
-		  (cond ((string< nx ny)
+		  (cond ((string<$ nx ny)
 			 t)
 			((string= nx ny)
 			 (cond ((eq nx ny) nil)
@@ -130,7 +172,8 @@
 
 (defun firstn (n lis)
   (subseq lis 0 n))
-  
+
+(declaim (inline fixnump))
 (defun fixnump (n)
   (typep n 'fixnum))
 
@@ -149,17 +192,19 @@
       `(setf (symbol-function ',sym) #',val)
       `(setf (get ',sym ',indic) ',val)))
 
-(defun sassq (item alist fun)
-  (or (assoc item alist :test #'eq)
-      (funcall fun)))
-
 ;; Find the N most significant or least significant bits of the
 ;; absolute value of X.  If N is positive, take the most significant;
 ;; otherwise, the least significant.
 (defun haipart (x n)
   (let ((x (abs x)))
     (if (< n 0)
-	(logand x (1- (ash 1 (- n))))
+	;; If the desired number of bits is larger than the actual
+	;; number, just return the number.  (Prevents gratuitously
+	;; generating a huge bignum if n is very large, as can happen
+	;; with bigfloats.)
+	(if (< (integer-length x) (- n))
+	    x
+	    (logand x (1- (ash 1 (- n)))))
 	(ash x (min (- n (integer-length x)) 0)))))
 
 ;; also correct but slower.
@@ -320,19 +365,34 @@
   ;; long-floats.
   #+nil
   (setf (ext:long-float-digits) 128)
+
+  ;; We want underflows not to signal errors.
+  (ext:without-package-lock ()
+    (setq sys::*inhibit-floating-point-underflow* t))
   )
 
+#+abcl
+(progn
+  ;; We want underflows not to signal errors
+  (when (fboundp (find-symbol "FLOAT-UNDERFLOW-MODE" "SYS"))
+    (funcall (find-symbol "FLOAT-UNDERFLOW-MODE" "SYS") nil))
+  )
+
+;; Make the maximum exponent larger for CMUCL.  Without this, cmucl
+;; will generate a continuable error when raising an integer to a
+;; power greater than this.
+#+cmu
+(setf ext::*intexp-maximum-exponent* 100000)
 ;;;; Setup the mapping from the Maxima 'flonum float type to a CL float type.
 ;;;;
-;;;; Add :flonum-log to *features* if you want flonum to be a
+;;;; Add :flonum-long to *features* if you want flonum to be a
 ;;;; long-float.  Or add :flonum-double-double if you want flonum to
 ;;;; be a double-double (currently only for CMUCL).  Otherwise, you
 ;;;; get double-float as the flonum type.
 ;;;;
 ;;;; Default double-float flonum.
-(eval-when (compile load eval)
-(setq *read-default-float-format* 'double-float)
-) ; eval-when
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (setq *read-default-float-format* 'double-float))
 
 #-(or flonum-long flonum-double-double)
 (progn
@@ -352,6 +412,7 @@
 ;; flonum be exactly the same as double-float, without bounds.
 #+clisp
 (deftype flonum (&optional low high)
+  (declare (ignorable low high))
   'double-float)
 
 (defconstant most-positive-flonum most-positive-double-float)
@@ -367,11 +428,10 @@
 #+flonum-long
 (progn
 ;;;; The Maxima 'flonum can be a CL 'long-float on the Scieneer CL or CLISP,
-;;;; but should be the same a 'double-float on other CL implementations.
+;;;; but should be the same as 'double-float on other CL implementations.
 
-(eval-when (compile load eval)
-(setq *read-default-float-format* 'long-float)
-) ; eval-when
+  (eval-when (:compile-toplevel :load-toplevel :execute)
+    (setq *read-default-float-format* 'long-float))
 
 ;; Tell Lisp the float type for a 'flonum.
 (deftype flonum (&optional low high)
@@ -398,9 +458,8 @@
 
 ;;;; The Maxima 'flonum can be a 'kernel:double-double-float on the CMU CL.
 
-(eval-when (compile load eval)
-(setq *read-default-float-format* 'kernel:double-double-float)
-) ; eval-when
+  (eval-when (:compile-toplevel :load-toplevel :execute)
+    (setq *read-default-float-format* 'kernel:double-double-float))
 
 ;; Tell Lisp the float type for a 'flonum.
 (deftype flonum (&optional low high)
@@ -440,9 +499,15 @@
 (defmacro float (x &optional (y 1e0))
   `(cl:float ,x ,y))
 
-;; DO-MERGE-ASYM moved here from nset.lisp so that it is defined before 
-;; it is referenced in compar.lisp.
+(defmacro with-collector (collector-sym &body forms)
+  (let ((acc (gensym)))
+    `(let ((,acc))
+       (flet ((,collector-sym (x) (push x ,acc)))
+         ,@forms
+         (nreverse ,acc)))))
 
+;; DO-MERGE-ASYM moved here from nset.lisp so that it is defined before
+;; it is referenced in compar.lisp.
 (defmacro do-merge-symm (list1 list2 eqfun lessfun bothfun onefun)
   ;; Like do-merge-asym, but calls onefun if an element appears in one but
   ;; not the other list, regardless of which list it appears in.
@@ -456,14 +521,10 @@
   ;; calls only1fun on each element that appears only in the first list;
   ;; calls only2fun on each element that appears only in the second list.
   ;; If both/only1/only2 fun are nil, treat as no-op.
-  ;; Initializes the variable "res" to nil; returns its value as the result.
   (let ((l1var (gensym))
 	(l2var (gensym)))
     `(do ((,l1var ,list1)
-	  (,l2var ,list2)
-	  res)
-	 ;; The variable RES is for the use of both/only1/only2-fun
-	 ;; do-merge-asym returns (nreverse res)
+	  (,l2var ,list2))
 	 ((cond ((null ,l1var)
 		 (if ,only2fun
 		     (while ,l2var
@@ -487,8 +548,7 @@
 		(t
 		 (if ,only2fun (funcall ,only2fun (car ,l2var)))
 		 (setq ,l2var (cdr ,l2var))
-		 nil))
-	  (reverse res)))))
+		 nil))))))
 
 ;;; Test
 ; (do-merge-asym '(a a a b c g h k l)
@@ -514,3 +574,49 @@
 ; one1 l
 ; nil
 
+;; Defines a function named NAME that checks that the number of
+;; arguments is correct.  If the number of actual arguments is
+;; incorrect, a maxima error is signaled.
+;;
+;; The required arguments is given by REQUIRED-ARG-LIST.  Allowed
+;; (maxima) keyword arguments is given by KEYWORD-ARG-LIST.
+;;
+;; The body of the function can refer to KEYLIST which is the list of
+;; maxima keyword arguments converted to lisp keyword arguments.
+
+(defmacro defun-checked (name ((&rest required-arg-list)
+			       &rest keyword-arg-list)
+			 &body body)
+  (let ((number-of-required-args (length required-arg-list))
+	(number-of-keyword-args (length keyword-arg-list))
+	(arg-list (gensym "ARG-LIST-"))
+	(helper-fun (gensym "REAL-FUN-"))
+	(options (gensym "OPTIONS-ARG-")))
+    `(defun ,name (&rest ,arg-list)
+       ;; Check that the required number of arguments is given and
+       ;; that we don't supply too many arguments.
+       ;;
+       ;; NOTE: The check when keyword args are given is a little too
+       ;; tight.  It's valid to have duplicate keyword args, but we
+       ;; disallow that if the number of arguments exceed the limit.
+       (when (or (> (length ,arg-list) ,(+ number-of-required-args number-of-keyword-args))
+		 (< (length ,arg-list) ,number-of-required-args))
+	 (merror (intl:gettext "~M arguments supplied to ~M: found ~M")
+		 (if (< (length ,arg-list) ,number-of-required-args)
+		     (intl:gettext "Too few")
+		     (if (> (length ,arg-list) ,(+ number-of-required-args
+						   number-of-keyword-args))
+			 (intl:gettext "Too many")
+			 (intl:gettext "Incorrect number of")))
+		 ',(if keyword-arg-list
+		       `((,name) ,@required-arg-list ((mlist simp) ,@keyword-arg-list))
+		       `((,name) ,@required-arg-list))
+		 (cons '(mlist) ,arg-list)))
+       (flet ((,helper-fun (,@required-arg-list
+			    ,@(when keyword-arg-list
+				`(&rest ,options)))
+		(let ,(when keyword-arg-list
+			`((keylist (lispify-maxima-keyword-options ,options
+								   ',keyword-arg-list))))
+	      ,@body)))
+	 (apply #',helper-fun ,arg-list)))))
