@@ -68,14 +68,13 @@
 (defvar *alt-display1d* nil)
 
 (defmfun displa (form &aux #+kcl(form form))
-  (if (or (not #.ttyoff) #.writefilep)
+  (if (not #.ttyoff)
       (cond ($display2d
 	     (cond (*alt-display2d* (apply *alt-display2d* form ()))
 		   (t
 		    (let ((displayp t)
 			  (linearray (if displayp (make-array 80.) linearray))
 			  (mratp (checkrat form))
-			  (#.writefilep #.writefilep)
 			  (maxht     1) (maxdp   0) (width   0)
 			  (height    0) (depth   0) (level   0) (size   2)
 			  (break     0) (right   0) (lines   1) bkpt
@@ -95,6 +94,12 @@
 		 (apply *alt-display1d* form ())
 		 (linear-displa form))))))
 
+(defmspec $with_default_2d_display (expr)
+  (let ((*alt-display2d* nil)
+        ($display2d t)
+        (mexpr (cons '(mprog) (cdr expr))))    
+    (meval mexpr)))
+
 ;;; (LINEAR-DISPLA <thing-to-display>)
 ;;;
 ;;; Display text linearly. This function should be usable in any case
@@ -124,7 +129,7 @@
     (cond ((atom form)
 	   (dimension-atom form result))
 	  ((and (atom (car form)) (setq form (cons '(mprogn) form)) nil))
-	  ((or (<= (lbp (caar form)) (rbp lop)) (> (lbp rop) (rbp (caar form))))
+	  ((or (<= (lbp (caar form)) (rbp lop)) (>= (lbp rop) (rbp (caar form))))
 	   (dimension-paren form result))
 	  ((member 'array (car form) :test #'eq) (dimension-array form result))
 	  ((safe-get (caar form) 'dimension)
@@ -1249,9 +1254,8 @@
 #+(or gcl clisp) (cond ((not (equal *query-io* *standard-input*)) (fresh-line)))
 #-(or gcl clisp) (cond ((not (interactive-stream-p *standard-input*)) (fresh-line)))
   (cond
-    ;; If output is turned off to the console and no WRITEFILE is taking
-    ;; place, then don't output anything.
-    ((and #.ttyoff (not #.writefilep)))
+    ;; If console output is disabled, don't output anything.
+    (#.ttyoff)
     ;; Constant 80. in this test appears to be the size of LINEARRAY.
     ((> (+ bkptht bkptdp) 80.)
      ;; I suppose we could reallocate LINEARRAY to some larger size and keep going here ...
